/* SmbOwn script for Samba Server for eCS (OS/2) */
   Version = '2.1.3'
   
/* Purpose: View the owner of (a) file(s) that has/have been accessed/created/
            changed on a Samba Server for eCS. Unlike the ls.exe utilities 
            owner data are not determined through kLibc, but directly from the
            extended attributes of the files. */
            
call _LoadOtherFuncs

call _InitTempDir

call _SambaInit

call _MasterPasswdRead

call _GroupRead

parse arg commandline
uCommandline = translate(CommandLine)

if Commandline = "" | Commandline = "/?" | uCommandline = "/H" then do
    say 'smbown [Filemask|/?|H]'
    say
    say '       Show user and group ownership information of (a) file(s)'
    exit 255
end

SubDirs = ''
I = 0

FileMask = strip(Commandline)


ok = SysFileTree(FileMask,Files.,'F'||Subdirs)

say left("Filename",40)' 'right("UID",4)' 'left("Username",14)' 'right("GID",4)' Groupname'
say copies("=",79)
do I = 1 to files.0
    HaveUId = 0
    HaveGID = 0
    Filename = substr(files.I,38)
    call charout , left(FileName,40)
    UIDOk=SysGetEA(FileName,'UID','UID')
    rUID = c2x(substr(UID,5))
    if UIDOk = 0 & rUID <> "" then do
        HaveUID = 1
        xUID = ""
        do j = 4 to 1 by -1
           xUID = xUID||substr(rUID,(2*j)-1,2)
           dUID = x2d(xUID)
        end
        CurUsername = "kLIBC User"
        do J = 1 to username.0
            if UID.J = dUID then do
                CurUsername = Username.J
                leave
            end
        end
        call charout ,' 'right(dUID,4)' 'left(CurUserName,14)
    end
    GIDOk=SysGetEA(FileName,'GID','GID')
    rGID = c2x(substr(GID,5))
    if GIDOk = 0 & c2x(substr(GID,5)) <> ""  then do
        HaveGID = 1
        xGID = ""
        do j = 4 to 1 by -1
           xGID = xGID||substr(rGID,(2*j)-1,2)
           dGID = x2d(xGID)
        end
        do j = 1 to GGID.0
           if dGID = GGID.j then CurGroup = groupname.J
        end
        call charout ,' 'right(dGID,4)' 'left(CurGroup,14)
    end
    if \HaveUID & \HaveGID then call charout , copies(d2c(8), 40)
    else say 
end 
exit 0

/* Common subroutines for several programs/scripts */

_LoadOtherFuncs: /* Load other REXX libraries */
    call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
    call SysLoadFuncs

    call RxFuncAdd 'VRLoadFuncs', 'VROBJ', 'VRLoadFuncs'
    call VRLoadFuncs
        
return

_InitTempDir: /* make sure we have a valid temporary directory */
    /* Get temporary directory */
    HaveNoTMPDIR  = 0
    TempDir       = translate(value('TMPDIR',,'OS2ENVIRONMENT'),'\','/')
    if TempDir = '' then do
        HaveNoTMPDIR  = 1
        TempDir = translate(value('TEMP',,'OS2ENVIRONMENT'),'\','/')
    end
    if TempDir = '' then TempDir = translate(value('TMP',,'OS2ENVIRONMENT'),'\','/')
    if TempDir = '' then TempDir = directory()
    if HaveNoTMPDIR then do
        say 'WARNING! Setting missing TMPDIR   variable to "'TempDir'".'
        ok = value('TMPDIR',TempDir,'OS2ENVIRONMENT')
    end    
    TempDir       = strip(TempDir,'T','\')||'\' /* make sure there is trailing "\" */
return

_SambaInit: /* Detect Samba components, initialize samba. stem */
    ETC      = value('ETC',,'OS2ENVIRONMENT')
    UnixRoot = value("UNIXROOT",,'OS2ENVIRONMENT')
    if UnixRoot = "" then do
        UnixRoot = left(ETC,length(ETC)-4)
        say 'WARNING! Setting missing UNIXROOT variable to "'UnixRoot'".'
        ok = value("UNIXROOT",UnixRoot,'OS2ENVIRONMENT')
    end
    UnixETC  = UnixRoot'\etc'
    samba.!masterpasswd = UnixEtc'\master.passwd'
    samba.!group        = UnixEtc'\group'

    /* get the path to this file */
    parse source . . script
    samba.!tools  = strip(filespec('D',script)||filespec('P',script),,'\')
    script = VRParseFileName(script,'N')

    samba.!pwd_mkdb = SysSearchPath("PATH","pwd_mkdb.exe")
    if samba.!pwd_mkdb = "" then do /* in current directory? */
        ok = SysFileTree('.\pwd_mkdb.exe', exist.,'FO')
        if exist.0 = 1 then samba.!pwd_mkdb = exist.1
    end
    if samba.!pwd_mkdb = "" then do /* in usr/sbin directory? */
        ok = SysFileTree(UnixRoot'\usr\sbin\pwd_mkdb.exe', exist.,'FO')
        if exist.0 = 1 then samba.!pwd_mkdb = exist.1
    end
    if samba.!pwd_mkdb = "" then do /* tools directory? */
        ok = SysFileTree(samba.!tools'\pwd_mkdb.exe', exist.,'FO')
        if exist.0 = 1 then samba.!pwd_mkdb = exist.1
    end
    if samba.!pwd_mkdb = "" then do /* in SMB_SBIN ? */
        samba.!pwd_mkdb = SysSearchPath("SMB_SBIN","pwd_mkdb.exe")
    end
return

_MasterPasswdRead: /* Read complete master.passwd into stems */
    
/* Structure of master.passwd file:
   Username:password:User-ID(UID):Group-ID(GID):Login-Class:Change pw in x seconds:Deactivate in x seconds:GECOS:HOME:SHELL
 */
    J = 0
    do until lines(samba.!masterpasswd) = 0
        userline = strip(linein(samba.!masterpasswd))
        if J = 0 & userline = "" then leave
        /* Skip comments */
        if left(userline,1) = "#" then iterate
        
        /* parse fields into stem variables */
        J = J + 1
    	parse var userline username.J':'password.J':'uid.J':'gid.J':'LoginClass.J':'pwchange.J':'deact.J':'gecos.J':'home.J':'shell.J
        Status.J =""	
        do K = 1 to J - 1
            if translate(Username.K) = translate(Username.J) then do
                status.J = "DUPLICATE"
                leave
            end
        end
    end
    ok = stream(samba.!masterpasswd,'c','close')
    
    /* set "stem roots" properly */
    username.0   = J
    password.0   = J
    uid.0        = J
    gid.0        = J
    loginclass.0 = J
    pwchange.0   = J
    deact.0      = J
    gecos.0      = J
    home.0       = J
    shell.0      = J
    status.0     = J
return

_GroupRead: /* Read complete groupfile into stems */
/* Structure of group file:
   groupname:*:GID:user,user,user, */

    Group512 = 0
    Group513 = 0
    Group515 = 0
    
    J = 0
    do until lines(samba.!group) = 0
        groupline = strip(linein(samba.!group))
        if J = 0 & groupline = "" then leave        
        
        /* Skip comments */
        if left(groupline,1) = "#" then iterate
        
        /* parse fields into stem variables */
        J = J + 1
        parse var groupline groupname.J':'gpasswd.J':'ggid.J':'gusers.J
        if ggid.J = 512 then Group512 = J
        if ggid.J = 513 then Group513 = J
        if ggid.J = 515 then Group515 = J
    end
    ok = stream(samba.!group,'c','close')

    /* set "stem roots" properly */
    groupname.0 = J
    gpasswd.0   = J; gpasswd. = '*' /* default value, unused */
    ggid.0      = J
    gusers.0    = J
return
